@@ -29,6 +29,7 @@
 #include "gkd-secret-types.h"
 #include "gkd-secret-util.h"
 
+#include "egg/egg-error.h"
 #include "egg/egg-secure-memory.h"
 
 #include "pkcs11/pkcs11i.h"
@@ -56,7 +57,7 @@ G_DEFINE_TYPE (GkdSecretChange, gkd_secret_change, GKD_SECRET_TYPE_PROMPT);
  */
 
 static void
-prepare_change_prompt (GkdSecretChange *self, GP11Object *collection)
+prepare_change_prompt (GkdSecretChange *self, GP11Object *collection, gboolean first)
 {
 	GError *error = NULL;
 	GkdPrompt *prompt;
@@ -69,7 +70,7 @@ prepare_change_prompt (GkdSecretChange *self, GP11Object *collection)
 
 	data = gp11_object_get_data (collection, CKA_LABEL, &n_data, &error);
 	if (!data) {
-		g_warning ("couldn't get label for collection: %s", error->message);
+		g_warning ("couldn't get label for collection: %s", egg_error_message (error));
 		g_clear_error (&error);
 	}
 
@@ -79,7 +80,8 @@ prepare_change_prompt (GkdSecretChange *self, GP11Object *collection)
 		label = g_strndup (data, n_data);
 	g_free (data);
 
-	gkd_prompt_reset (prompt);
+	/* Hard reset on first prompt, soft thereafter */
+	gkd_prompt_reset (prompt, first);
 
 	gkd_prompt_set_title (prompt, _("Change Keyring Password"));
 
@@ -131,7 +133,7 @@ gkd_secret_change_prompt_ready (GkdSecretPrompt *prompt)
 	}
 
 	if (!gkd_prompt_has_response (GKD_PROMPT (prompt))) {
-		prepare_change_prompt (self, collection);
+		prepare_change_prompt (self, collection, TRUE);
 		return;
 	}
 
@@ -149,7 +151,7 @@ gkd_secret_change_prompt_ready (GkdSecretPrompt *prompt)
 
 	/* The original password was incorrect */
 	} else if (dbus_error_has_name (&derr, INTERNAL_ERROR_DENIED)) {
-		prepare_change_prompt (self, collection);
+		prepare_change_prompt (self, collection, FALSE);
 		set_warning_wrong (self);
 
 	/* Other failures */
@@ -303,10 +305,10 @@ cleanup:
 		gp11_attributes_unref (attrs);
 
 	if (!result && error) {
-		if (error->code == CKR_USER_NOT_LOGGED_IN)
+		if (g_error_matches (error, GP11_ERROR, CKR_USER_NOT_LOGGED_IN))
 			dbus_set_error (derr, INTERNAL_ERROR_DENIED, "The original password was invalid");
 		else
-			g_warning ("failure occurred while changing password: %s", error->message);
+			g_warning ("failure occurred while changing password: %s", egg_error_message (error));
 	}
 
 	if (!result && !dbus_error_is_set (derr))
