@@ -23,11 +23,14 @@
 
 #include "gkd-login.h"
 
+#include "egg/egg-error.h"
 #include "egg/egg-secure-memory.h"
 
 #include "pkcs11/gkd-pkcs11.h"
 #include "pkcs11/pkcs11i.h"
 
+#include <glib/gi18n.h>
+
 #include <string.h>
 
 static gint unlock_failures = 0;
@@ -38,6 +41,18 @@ note_that_unlock_failed (void)
 	g_atomic_int_inc (&unlock_failures);
 }
 
+static void
+note_that_unlock_succeeded (void)
+{
+	g_atomic_int_set (&unlock_failures, 0);
+}
+
+gboolean
+gkd_login_did_unlock_fail (void)
+{
+	return g_atomic_int_get (&unlock_failures) ? TRUE : FALSE;
+}
+
 static GP11Module*
 module_instance (void)
 {
@@ -62,7 +77,9 @@ open_and_login_session (GP11Slot *slot, CK_USER_TYPE user_type, GError **error)
 	session = gp11_slot_open_session (slot, CKF_RW_SESSION, error);
 	if (session != NULL) {
 		if (!gp11_session_login (session, user_type, NULL, 0, error)) {
-			if ((*error)->code != CKR_USER_ALREADY_LOGGED_IN) {
+			if (g_error_matches (*error, GP11_ERROR, CKR_USER_ALREADY_LOGGED_IN)) {
+				g_clear_error (error);
+			} else {
 				g_object_unref (session);
 				session = NULL;
 			}
@@ -103,8 +120,8 @@ lookup_login_session (GP11Module *module)
 	g_return_val_if_fail (slot, NULL);
 
 	session = open_and_login_session (slot, CKU_USER, &error);
-	if (session == NULL) {
-		g_warning ("couldn't open pkcs11 session for login: %s", error->message);
+	if (error) {
+		g_warning ("couldn't open pkcs11 session for login: %s", egg_error_message (error));
 		g_clear_error (&error);
 	}
 
@@ -130,7 +147,7 @@ lookup_login_keyring (GP11Session *session)
 	                                     GP11_INVALID);
 
 	if (error) {
-		g_warning ("couldn't search for login keyring: %s", error->message);
+		g_warning ("couldn't search for login keyring: %s", egg_error_message (error));
 		g_clear_error (&error);
 		return NULL;
 	}
@@ -151,13 +168,18 @@ static GP11Object*
 create_login_keyring (GP11Session *session, GP11Object *cred, GError **error)
 {
 	GP11Object *login;
+	const gchar *label;
 
 	g_return_val_if_fail (GP11_IS_SESSION (session), NULL);
 	g_return_val_if_fail (GP11_IS_OBJECT (cred), NULL);
 
+	/* TRANSLATORS: This is the display label for the login keyring */
+	label = _("Login");
+
 	login = gp11_session_create_object (session, error,
 	                                    CKA_CLASS, GP11_ULONG, CKO_G_COLLECTION,
 	                                    CKA_ID, (gsize)5, "login",
+	                                    CKA_LABEL, strlen (label), label,
 	                                    CKA_G_CREDENTIAL, GP11_ULONG, gp11_object_get_handle (cred),
 	                                    CKA_TOKEN, GP11_BOOLEAN, TRUE,
 	                                    GP11_INVALID);
@@ -219,19 +241,23 @@ unlock_or_create_login (GP11Module *module, const gchar *master)
 
 	/* Failure, bad password? */
 	if (cred == NULL) {
-		if (login && error->code == CKR_PIN_INCORRECT)
+		if (login && g_error_matches (error, GP11_ERROR, CKR_PIN_INCORRECT))
 			note_that_unlock_failed ();
 		else
-			g_warning ("couldn't create login credential: %s", error->message);
+			g_warning ("couldn't create login credential: %s", egg_error_message (error));
 		g_clear_error (&error);
 
 	/* Non login keyring, create it */
 	} else if (!login) {
 		login = create_login_keyring (session, cred, &error);
-		if (login == NULL) {
-			g_warning ("couldn't create login keyring: %s", error->message);
+		if (login == NULL && error) {
+			g_warning ("couldn't create login keyring: %s", egg_error_message (error));
 			g_clear_error (&error);
 		}
+
+	/* The unlock succeeded yay */
+	} else {
+		note_that_unlock_succeeded ();
 	}
 
 	if (cred)
@@ -270,8 +296,9 @@ init_pin_for_uninitialized_slots (GP11Module *module, const gchar *master)
 					                         "serial-number", info->serial_number,
 					                         NULL);
 				} else {
-					if (error->code != CKR_FUNCTION_NOT_SUPPORTED)
-						g_warning ("couldn't initialize slot with master password: %s", error->message);
+					if (!g_error_matches (error, GP11_ERROR, CKR_FUNCTION_NOT_SUPPORTED))
+						g_warning ("couldn't initialize slot with master password: %s",
+						           egg_error_message (error));
 					g_clear_error (&error);
 				}
 				g_object_unref (session);
@@ -325,19 +352,21 @@ change_or_create_login (GP11Module *module, const gchar *original, const gchar *
 	/* Create the new credential we'll be changing to */
 	mcred = create_credential (session, NULL, master, &error);
 	if (mcred == NULL) {
-		g_warning ("couldn't create new login credential: %s", error->message);
+		g_warning ("couldn't create new login credential: %s", egg_error_message (error));
 		g_clear_error (&error);
 
 	/* Create original credentials */
 	} else if (login) {
 		ocred = create_credential (session, login, original, &error);
 		if (ocred == NULL) {
-			if (error->code == CKR_PIN_INCORRECT) {
+			if (g_error_matches (error, GP11_ERROR, CKR_PIN_INCORRECT)) {
 				g_message ("couldn't change login master password, "
-				           "original password was wrong: %s", error->message);
+				           "original password was wrong: %s",
+				           egg_error_message (error));
 				note_that_unlock_failed ();
 			} else {
-				g_warning ("couldn't create original login credential: %s", error->message);
+				g_warning ("couldn't create original login credential: %s",
+				           egg_error_message (error));
 			}
 			g_clear_error (&error);
 		}
@@ -347,7 +376,7 @@ change_or_create_login (GP11Module *module, const gchar *original, const gchar *
 	if (!login && mcred) {
 		login = create_login_keyring (session, mcred, &error);
 		if (login == NULL) {
-			g_warning ("couldn't create login keyring: %s", error->message);
+			g_warning ("couldn't create login keyring: %s", egg_error_message (error));
 			g_clear_error (&error);
 		} else {
 			success = TRUE;
@@ -358,7 +387,7 @@ change_or_create_login (GP11Module *module, const gchar *original, const gchar *
 		if (!gp11_object_set (login, &error,
 		                      CKA_G_CREDENTIAL, GP11_ULONG, gp11_object_get_handle (mcred),
 		                      GP11_INVALID)) {
-			g_warning ("couldn't change login master password: %s", error->message);
+			g_warning ("couldn't change login master password: %s", egg_error_message (error));
 			g_clear_error (&error);
 		} else {
 			success = TRUE;
@@ -409,8 +438,10 @@ set_pin_for_any_slots (GP11Module *module, const gchar *original, const gchar *m
 					                         "serial-number", info->serial_number,
 					                         NULL);
 				} else {
-					if (error->code != CKR_PIN_INCORRECT && error->code != CKR_FUNCTION_NOT_SUPPORTED)
-						g_warning ("couldn't change slot master password: %s", error->message);
+					if (!g_error_matches (error, GP11_ERROR, CKR_PIN_INCORRECT) &&
+					    !g_error_matches (error, GP11_ERROR, CKR_FUNCTION_NOT_SUPPORTED))
+						g_warning ("couldn't change slot master password: %s",
+						           egg_error_message (error));
 					g_clear_error (&error);
 				}
 				g_object_unref (session);
@@ -514,7 +545,7 @@ find_login_keyring_item (GP11Session *session, GP11Attribute *fields)
 	                                     GP11_INVALID);
 
 	if (!search) {
-		g_warning ("couldn't create search for login keyring: %s", error->message);
+		g_warning ("couldn't create search for login keyring: %s", egg_error_message (error));
 		g_clear_error (&error);
 		return NULL;
 	}
@@ -526,7 +557,7 @@ find_login_keyring_item (GP11Session *session, GP11Attribute *fields)
 	g_object_unref (search);
 
 	if (data == NULL) {
-		g_warning ("couldn't read search in login keyring: %s", error->message);
+		g_warning ("couldn't read search in login keyring: %s", egg_error_message (error));
 		g_clear_error (&error);
 		return NULL;
 	}
@@ -545,19 +576,88 @@ find_login_keyring_item (GP11Session *session, GP11Attribute *fields)
 	return item;
 }
 
+static GP11Attributes*
+attach_make_attributes_va (GP11Session *session, const gchar *label,
+                           const gchar *first, va_list va)
+{
+	GP11Attributes *attrs;
+	GP11Attribute fields;
+	gchar *display_name;
+	GP11Object* item;
+	GError *error = NULL;
+	gpointer value;
+	gsize n_value;
+
+	attrs = gp11_attributes_new ();
+
+	gp11_attribute_init_empty (&fields, CKA_G_FIELDS);
+	string_attribute_list_va (va, first, &fields);
+
+	/*
+	 * If there already is such an item, then include its identifier.
+	 * What this does is overwrite that item, rather than creating new.
+	 */
+	item = find_login_keyring_item (session, &fields);
+	if (item) {
+		value = gp11_object_get_data (item, CKA_ID, &n_value, &error);
+		if (value != NULL) {
+			gp11_attributes_add_data (attrs, CKA_ID, value, n_value);
+			g_free (value);
+		} else {
+			g_warning ("couldn't retrieve id for previous login item: %s",
+			           egg_error_message (error));
+			g_clear_error (&error);
+		}
+		g_object_unref (item);
+	}
+
+	if (label == NULL)
+		label = _("Unnamed");
+
+	display_name = g_strdup_printf (_("Unlock password for: %s"), label);
+	gp11_attributes_add_string (attrs, CKA_LABEL, display_name);
+
+	gp11_attributes_add_boolean (attrs, CKA_TOKEN, TRUE);
+	gp11_attributes_add_ulong (attrs, CKA_CLASS, CKO_SECRET_KEY);
+	gp11_attributes_add_data (attrs, CKA_G_COLLECTION, "login", (gsize)5);
+	gp11_attributes_add (attrs, &fields);
+
+	gp11_attribute_clear (&fields);
+	return attrs;
+}
+
+GP11Attributes*
+gkd_login_attach_make_attributes (const gchar *label, const gchar *first, ...)
+{
+	GP11Attributes *attrs;
+	GP11Session *session;
+	GP11Module *module;
+	va_list va;
+
+	module = module_instance ();
+	session = lookup_login_session (module);
+
+	va_start (va, first);
+	attrs = attach_make_attributes_va (session, label, first, va);
+	va_end (va);
+
+	g_object_unref (session);
+	g_object_unref (module);
+
+	return attrs;
+}
+
 void
-gkd_login_attach_secret (const gchar *display_name, const gchar *secret,
+gkd_login_attach_secret (const gchar *label, const gchar *secret,
                          const gchar *first, ...)
 {
 	GError *error = NULL;
-	GP11Attribute fields;
 	GP11Session *session;
 	GP11Module *module;
-	GP11Object* item;
+	GP11Attributes *attrs;
+	GP11Object *item;
 	va_list va;
 
-	if (display_name == NULL)
-		display_name = "";
 	if (secret == NULL)
 		secret = "";
 
@@ -565,33 +665,20 @@ gkd_login_attach_secret (const gchar *display_name, const gchar *secret,
 	session = lookup_login_session (module);
 
 	va_start(va, first);
-	gp11_attribute_init_empty (&fields, CKA_G_FIELDS);
-	string_attribute_list_va (va, first, &fields);
+	attrs = attach_make_attributes_va (session, label, first, va);
 	va_end(va);
 
-	item = find_login_keyring_item (session, &fields);
-	if (item) {
-		gp11_object_set (item, &error,
-		                 CKA_LABEL, strlen (display_name), display_name,
-		                 CKA_VALUE, strlen (secret), secret,
-		                 GP11_INVALID);
-	} else {
-		item = gp11_session_create_object (session, &error,
-		                                   CKA_CLASS, GP11_ULONG, CKO_SECRET_KEY,
-		                                   CKA_LABEL, strlen (display_name), display_name,
-		                                   CKA_VALUE, strlen (secret), secret,
-		                                   CKA_G_COLLECTION, (gsize)5, "login",
-		                                   CKA_G_FIELDS, fields.length, fields.value,
-		                                   GP11_INVALID);
-	}
-
+	gp11_attributes_add_string (attrs, CKA_VALUE, secret);
+	item = gp11_session_create_object_full (session, attrs, NULL, &error);
 	if (error != NULL) {
-		g_warning ("couldn't store secret in login keyring: %s", error->message);
+		g_warning ("couldn't store secret in login keyring: %s", egg_error_message (error));
 		g_clear_error (&error);
 	}
 
 	if (item)
 		g_object_unref (item);
+
+	gp11_attributes_unref (attrs);
 	g_object_unref (session);
 	g_object_unref (module);
 }
@@ -655,8 +742,9 @@ gkd_login_remove_secret (const gchar *first, ...)
 	item = find_login_keyring_item (session, &fields);
 	if (item != NULL) {
 		if (!gp11_object_destroy (item, &error)) {
-			if (error->code != CKR_OBJECT_HANDLE_INVALID)
-				g_warning ("couldn't remove stored secret from login keyring: %s", error->message);
+			if (!g_error_matches (error, GP11_ERROR, CKR_OBJECT_HANDLE_INVALID))
+				g_warning ("couldn't remove stored secret from login keyring: %s",
+				           egg_error_message (error));
 			g_clear_error (&error);
 		}
 		g_object_unref (item);
@@ -665,22 +753,3 @@ gkd_login_remove_secret (const gchar *first, ...)
 	g_object_unref (session);
 	g_object_unref (module);
 }
-
-GP11Attributes*
-gkd_login_attributes_for_secret (const gchar *first, ...)
-{
-	GP11Attributes *attrs;
-	GP11Attribute *fields;
-	va_list va;
-
-	attrs = gp11_attributes_newv (CKA_CLASS, GP11_ULONG, CKO_SECRET_KEY,
-	                              CKA_G_COLLECTION, (gsize)5, "login",
-	                              GP11_INVALID);
-
-	va_start(va, first);
-	fields = gp11_attributes_add_empty (attrs, CKA_G_FIELDS);
-	string_attribute_list_va (va, first, fields);
-	va_end(va);
-
-	return attrs;
-}
